/*****************************************************************************
* | File      	:   LCD_1in14_test.c
* | Author      :   Waveshare team
* | Function    :   2.9inch e-paper V2 test demo
* | Info        :
*----------------
* |	This version:   V1.0
* | Date        :   2021-01-20
* | Info        :
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documnetation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to  whom the Software is
# furished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in
# all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS OR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
# THE SOFTWARE.
#
******************************************************************************/
#include "LCD_IMU_Test.h"
#include "LCD_1in14.h"
#include "mpu9250.h"
#include "lps22hb.h"

bool reserved_addr(uint8_t addr) {
return (addr & 0x78) == 0 || (addr & 0x78) == 0x78;
}

int LCD_1in14_test(void)
{
    DEV_Delay_ms(20);
    printf("LCD_1in14_test Demo\r\n");
    if(DEV_Module_Init()!=0){
        return -1;
    }

        /* LCD Init */
    printf("1.14inch LCD demo...\r\n");
    LCD_1IN14_Init(HORIZONTAL);
    // LCD_1IN14_Clear(0x0000);

    //LCD_SetBacklight(1023);
    UDOUBLE Imagesize = LCD_1IN14_HEIGHT*LCD_1IN14_WIDTH*2;
    UWORD *BlackImage;
    if((BlackImage = (UWORD *)malloc(Imagesize)) == NULL) {
        printf("Failed to apply for black memory...\r\n");
        exit(0);
    }
    // /*1.Create a new image cache named IMAGE_RGB and fill it with white
    Paint_NewImage(BlackImage,LCD_1IN14.WIDTH,LCD_1IN14.HEIGHT, 0, WHITE, 16);
    Paint_Clear(WHITE);
    Paint_SetRotate(ROTATE_0);
    Paint_Clear(WHITE);
#if 1 
	IMU_EN_SENSOR_TYPE enMotionSensorType;
	IMU_ST_ANGLES_DATA stAngles;
	IMU_ST_SENSOR_DATA stGyroRawData;
	IMU_ST_SENSOR_DATA stAccelRawData;
	IMU_ST_SENSOR_DATA stMagnRawData;
    float PRESS_DATA=0;
    float TEMP_DATA=0;
    uint8_t u8Buf[3];
	imuInit(&enMotionSensorType);
	if(IMU_EN_SENSOR_TYPE_MPU9250 == enMotionSensorType)
	{
		printf("Motion sersor is ICM-20948\n" );
	}
	else
	{
		printf("Motion sersor NULL\n");
	}
	if (!LPS22HB_INIT()){
		printf("LPS22HB Init Error\n");
		return 0;
	}
    Paint_Clear(WHITE);
    Paint_DrawString_EN(55, 1, "Waveshare", &Font20, WHITE, BLUE);
    Paint_DrawString_EN(35, 20, "Pico-10DOF-IMU", &Font16, WHITE, BRED);
    Paint_DrawString_EN(196, 40, "Deg", &Font16, WHITE , BROWN);
    Paint_DrawString_EN(196, 56, "Deg", &Font16, WHITE , BLACK);
    Paint_DrawString_EN(196, 71, "Deg", &Font16, WHITE , GBLUE);
    Paint_DrawString_EN(196, 87, "hPa", &Font16, WHITE , RED);
    Paint_DrawString_EN(209, 103, "C", &Font16, WHITE , GREEN);
    Paint_DrawString_EN(1, 40, "roll", &Font16, WHITE, BROWN);
    Paint_DrawString_EN(1, 56, "pitch", &Font16, WHITE, BLACK);
    Paint_DrawString_EN(1, 71, "yaw", &Font16, WHITE, GBLUE);
    Paint_DrawString_EN(1, 87, "pressure", &Font16, WHITE, RED);
    Paint_DrawString_EN(1, 103, "temperature", &Font16, WHITE, GREEN);
    while(1){ 
            
		LPS22HB_START_ONESHOT();
        if((I2C_readByte(LPS_STATUS)&0x01)==0x01)   //a new pressure data is generated
        {
            u8Buf[0]=I2C_readByte(LPS_PRESS_OUT_XL);
            u8Buf[1]=I2C_readByte(LPS_PRESS_OUT_L);
            u8Buf[2]=I2C_readByte(LPS_PRESS_OUT_H);
            PRESS_DATA=(float)((u8Buf[2]<<16)+(u8Buf[1]<<8)+u8Buf[0])/4096.0f;
        }
        if((I2C_readByte(LPS_STATUS)&0x02)==0x02)   // a new pressure data is generated
        {
            u8Buf[0]=I2C_readByte(LPS_TEMP_OUT_L);
            u8Buf[1]=I2C_readByte(LPS_TEMP_OUT_H);
            TEMP_DATA=(float)((u8Buf[1]<<8)+u8Buf[0])/100.0f;
        }
        
		imuDataGet( &stAngles, &stGyroRawData, &stAccelRawData, &stMagnRawData);

        Paint_ClearWindow(148,40,194,60,WHITE);
		if(0> stAngles.fRoll){
			Paint_DrawString_EN(148, 40, "-", &Font16, WHITE , BROWN);
			int roll = 0-stAngles.fRoll;
			if(0==roll){
				Paint_DrawString_EN(158, 40, "0", &Font16, WHITE , BROWN);
			}else{
				Paint_DrawNum(158, 40, roll, &Font16, WHITE , BROWN);
			}
		}else {
			Paint_DrawString_EN(148, 40, "+", &Font16, WHITE , BROWN);
			int roll = stAngles.fRoll;
			if(0==roll){
				Paint_DrawString_EN(158, 40, "0", &Font16, WHITE , BROWN);
			}else{
				Paint_DrawNum(158, 40, roll, &Font16, WHITE , BROWN);
			}
		}
                
        Paint_ClearWindow(148,56,194,76,WHITE);
        if(0>stAngles.fPitch){
			Paint_DrawString_EN(148, 56, "-", &Font16, WHITE , BLACK);
			int pitch = 0-stAngles.fPitch;
			if(0==pitch){
				Paint_DrawString_EN(158, 56, "0", &Font16, WHITE , BLACK);
			}else{
				Paint_DrawNum(158, 56, pitch, &Font16, WHITE , BLACK);
			}
		}else{ 
			Paint_DrawString_EN(148, 56, "+", &Font16, WHITE , BLACK);
			int pitch = stAngles.fPitch;
			if(0==pitch){
				Paint_DrawString_EN(158, 56, "0", &Font16, WHITE , BLACK);
			}else{
				Paint_DrawNum(158, 56, pitch , &Font16, WHITE , BLACK);
			}
		}
        Paint_ClearWindow(148,71,194,91,WHITE);
		if(0>stAngles.fYaw){
			Paint_DrawString_EN(148, 71, "-", &Font16, WHITE , GBLUE);
			int yaw = 0-stAngles.fYaw;
			if(0==yaw){
				Paint_DrawString_EN(158, 71, "0", &Font16, WHITE , GBLUE);
			}else{
				Paint_DrawNum(158, 71, yaw, &Font16, WHITE , GBLUE);
			}
		}else{
			Paint_DrawString_EN(148, 71, "+", &Font16, WHITE , GBLUE);
			int yaw = stAngles.fYaw;
			if(0==yaw){
				Paint_DrawString_EN(158, 71, "0", &Font16, WHITE , GBLUE);
			}else{
				Paint_DrawNum(158, 71, yaw , &Font16, WHITE , GBLUE);
			}
		}
        Paint_ClearWindow(148,87,194,107,WHITE);
		Paint_DrawNum(148, 87, PRESS_DATA , &Font16, WHITE , RED);
        Paint_ClearWindow(148,103,208,123,WHITE);
		int temp= TEMP_DATA*100;
		if(0>TEMP_DATA){ 
			Paint_DrawString_EN(148, 103, "-", &Font16, WHITE , GREEN);
			int tem = 0-TEMP_DATA;
			Paint_DrawNum(158, 103, tem , &Font16, WHITE , GREEN);
		}else{
			Paint_DrawString_EN(148, 103, "+", &Font16, WHITE , GREEN);
			Paint_DrawNum(158, 103, TEMP_DATA, &Font16, WHITE , GREEN);
			Paint_DrawString_EN(178, 103, ".", &Font16, WHITE, GREEN);
            if(0 == temp/10%10)
            Paint_DrawString_EN(188, 103, "0", &Font16, WHITE , GREEN);
            else
			Paint_DrawNum(188, 103, temp/10%10, &Font16, WHITE , GREEN);
            if(0 == temp%10)
            Paint_DrawString_EN(198, 103, "0", &Font16, WHITE , GREEN);
            else
			Paint_DrawNum(198, 103, temp%10, &Font16, WHITE , GREEN);
		}
        
        LCD_1IN14_Display(BlackImage);
#endif 
    
}
    /* Module Exit */
    free(BlackImage);
    BlackImage = NULL;
    
    
    DEV_Module_Exit();
    return 0;
}
