#include "mpu9250.h"
#include <string.h>

#define I2C_PORT i2c1
int16_t magn[3];
int16_t accel[3], gyro[3];
unsigned char BUF[10];
MPU9250_TypeDef MPU9250_Offset={0};
MPU9250_TypeDef_Off MPU9250_Magn_Offset={0};
#ifdef __cplusplus
extern "C" {
#endif

char I2C_ReadOneByte(uint8_t addr, char reg)
{
	char buf;
	i2c_write_blocking(I2C_PORT,addr,&reg,1,true);
	i2c_read_blocking(I2C_PORT,addr,&buf,1,false);
	return buf;
}

void I2C_WriteOneByte(uint8_t addr, char reg, char value)
{
	char buf[]={reg,value};
	i2c_write_blocking(I2C_PORT,addr,buf,2,false);
}
/******************************************************************************
 * MPU9250 sensor device                                                     *
 ******************************************************************************/
/**
  * @brief  Initializes MPU9250
  * @param  None
  * @retval None
  */
void MPU9250_Init(void)
{
  I2C_WriteOneByte(GYRO_ADDRESS,PWR_MGMT_1, 0x00);
  I2C_WriteOneByte(GYRO_ADDRESS,SMPLRT_DIV, 0x07);
  I2C_WriteOneByte(GYRO_ADDRESS,CONFIG, 0x06);
  I2C_WriteOneByte(GYRO_ADDRESS,GYRO_CONFIG, 0x10);
  I2C_WriteOneByte(GYRO_ADDRESS,ACCEL_CONFIG, 0x01);
  
  sleep_ms(10);
  MPU9250_InitGyrOffset();
}

/**
  * @brief Get accelerometer datas
  * @param  None
  * @retval None
  */
void MPU9250_READ_ACCEL(void)
{ 
   uint8_t i;
   int16_t InBuffer[3] = {0}; 
   static int32_t OutBuffer[3] = {0};
   static MPU9250_AvgTypeDef MPU9250_Filter[3];

   BUF[0]=I2C_ReadOneByte(ACCEL_ADDRESS,ACCEL_XOUT_L); 
   BUF[1]=I2C_ReadOneByte(ACCEL_ADDRESS,ACCEL_XOUT_H);
   InBuffer[0]= (BUF[1]<<8)|BUF[0];

   BUF[2]=I2C_ReadOneByte(ACCEL_ADDRESS,ACCEL_YOUT_L);
   BUF[3]=I2C_ReadOneByte(ACCEL_ADDRESS,ACCEL_YOUT_H);
   InBuffer[1]= (BUF[3]<<8)|BUF[2];
             
   BUF[4]=I2C_ReadOneByte(ACCEL_ADDRESS,ACCEL_ZOUT_L);
   BUF[5]=I2C_ReadOneByte(ACCEL_ADDRESS,ACCEL_ZOUT_H);
   InBuffer[2]= (BUF[5]<<8)|BUF[4];            
   
   for(i = 0; i < 3; i ++)  
   {
      MPU9250_CalAvgValue(&MPU9250_Filter[i].Index, MPU9250_Filter[i].AvgBuffer, InBuffer[i], OutBuffer + i);
   }
   accel[0] = *(OutBuffer + 0);
   accel[1] = *(OutBuffer + 1);
   accel[2] = *(OutBuffer + 2); 
}

/**
  * @brief Get gyroscopes datas
  * @param  None
  * @retval None
  */
void MPU9250_READ_GYRO(void)
{ 
   uint8_t i;
   int16_t InBuffer[3] = {0}; 
   static int32_t OutBuffer[3] = {0};
   static MPU9250_AvgTypeDef MPU9250_Filter[3];

   BUF[0]=I2C_ReadOneByte(GYRO_ADDRESS,GYRO_XOUT_L); 
   BUF[1]=I2C_ReadOneByte(GYRO_ADDRESS,GYRO_XOUT_H);
   InBuffer[0]= (BUF[1]<<8)|BUF[0];
   
   BUF[2]=I2C_ReadOneByte(GYRO_ADDRESS,GYRO_YOUT_L);
   BUF[3]=I2C_ReadOneByte(GYRO_ADDRESS,GYRO_YOUT_H);
   InBuffer[1] = (BUF[3]<<8)|BUF[2];
    
   BUF[4]=I2C_ReadOneByte(GYRO_ADDRESS,GYRO_ZOUT_L);
   BUF[5]=I2C_ReadOneByte(GYRO_ADDRESS,GYRO_ZOUT_H);
   InBuffer[2] = (BUF[5]<<8)|BUF[4];  

   for(i = 0; i < 3; i ++)  
   {
      MPU9250_CalAvgValue(&MPU9250_Filter[i].Index, MPU9250_Filter[i].AvgBuffer, InBuffer[i], OutBuffer + i);
   }
   gyro[0] = *(OutBuffer + 0) - MPU9250_Offset.X;
   gyro[1] = *(OutBuffer + 1) - MPU9250_Offset.Y;
   gyro[2] = *(OutBuffer + 2) - MPU9250_Offset.Z;
}

/**
  * @brief Get compass datas
  * @param  None
  * @retval None
  */
void MPU9250_READ_MAG(void)
{ 
   uint8_t i;
   int16_t InBuffer[3] = {0}; 
   static int32_t OutBuffer[3] = {0};
   static MPU9250_AvgTypeDef MPU9250_Filter[3];

    I2C_WriteOneByte(GYRO_ADDRESS,0x37,0x02);//turn on Bypass Mode 
    sleep_ms(10);
    I2C_WriteOneByte(MAG_ADDRESS,0x0A,0x01);  
    sleep_ms(10);
    BUF[0]=I2C_ReadOneByte (MAG_ADDRESS,MAG_XOUT_L);
    BUF[1]=I2C_ReadOneByte (MAG_ADDRESS,MAG_XOUT_H);
    InBuffer[1] =(BUF[1]<<8)|BUF[0];

    BUF[2]=I2C_ReadOneByte(MAG_ADDRESS,MAG_YOUT_L);
    BUF[3]=I2C_ReadOneByte(MAG_ADDRESS,MAG_YOUT_H);
    InBuffer[0] = (BUF[3]<<8)|BUF[2];
    
    BUF[4]=I2C_ReadOneByte(MAG_ADDRESS,MAG_ZOUT_L);
    BUF[5]=I2C_ReadOneByte(MAG_ADDRESS,MAG_ZOUT_H);
    InBuffer[2] = (BUF[5]<<8)|BUF[4]; 
    InBuffer[2] = -InBuffer[2];
   
   for(i = 0; i < 3; i ++)  
   {
      MPU9250_CalAvgValue(&MPU9250_Filter[i].Index, MPU9250_Filter[i].AvgBuffer, InBuffer[i], OutBuffer + i);
   } 
      magn[0] = *(OutBuffer + 0)-MPU9250_Magn_Offset.X_Off_Err;
      magn[1] = *(OutBuffer + 1)-MPU9250_Magn_Offset.Y_Off_Err;
      magn[2] = *(OutBuffer + 2)-MPU9250_Magn_Offset.Z_Off_Err;
}

/**
  * @brief  Check MPU9250,ensure communication succeed
  * @param  None
  * @retval true: communicate succeed
  *               false: communicate fualt 
  */
bool MPU9250_Check(void) 
{   
    if(WHO_AM_I_VAL == I2C_ReadOneByte(0x68, WHO_AM_I))
    {
      return true;
    }
    else 
    {
      return false;
    }
}

/**
  * @brief  Digital filter
  * @param *pIndex:
  * @param *pAvgBuffer:
  * @param InVal:
  * @param pOutVal:
  *
  * @retval None
  *               
  */
void MPU9250_CalAvgValue(uint8_t *pIndex, int16_t *pAvgBuffer, int16_t InVal, int32_t *pOutVal)
{ 
  uint8_t i;
  
  *(pAvgBuffer + ((*pIndex) ++)) = InVal;
    *pIndex &= 0x07;
    
    *pOutVal = 0;
  for(i = 0; i < 8; i ++) 
    {
      *pOutVal += *(pAvgBuffer + i);
    }
    *pOutVal >>= 3;
}
/**
  * @brief  Initializes gyroscopes offset
  * @param  None
  * @retval None
  */
void MPU9250_InitGyrOffset(void)
{
  uint8_t i;
  int32_t TempGx = 0, TempGy = 0, TempGz = 0;
  
  for(i = 0; i < 32; i ++)
  {
    MPU9250_READ_GYRO();
    
    TempGx += gyro[0];
    TempGy += gyro[1];
    TempGz += gyro[2];

    sleep_ms(1);
  }

  MPU9250_Offset.X = TempGx >> 5;
  MPU9250_Offset.Y = TempGy >> 5;
  MPU9250_Offset.Z = TempGz >> 5;

}

void calibrateMagn(void)
{
  int16_t temp[9];
  printf("keep 10dof-imu device horizontal and it will read x y z axis offset value after 4 seconds\n");
  sleep_ms(4000);
  printf("start read all axises offset value\n");
  MPU9250_READ_MAG();
  temp[0] = magn[0];
  temp[1] = magn[1];
  temp[2] = magn[2];
  
  printf("rotate z axis 180 degrees and it will read all axises offset value after 4 seconds\n");
  sleep_ms(4000);
  printf("start read all axises offset value\n");
  MPU9250_READ_MAG();
  temp[3] = magn[0];
  temp[4] = magn[1];
  temp[5] = magn[2];

  printf("flip 10dof-imu device and keep it horizontal and it will read all axises offset value after 4 seconds\n");
  sleep_ms(4000);
  printf("start read all axises offset value\n");
  MPU9250_READ_MAG();
  temp[6] = magn[0];
  temp[7] = magn[1];
  temp[8] = magn[2];

  MPU9250_Magn_Offset.X_Off_Err = (temp[0]+temp[3])/2;
  MPU9250_Magn_Offset.Y_Off_Err = (temp[1]+temp[4])/2;
  MPU9250_Magn_Offset.Z_Off_Err = (temp[5]+temp[8])/2;
}

/******************************************************************************
 * IMU module                                                                 *
 ******************************************************************************/
#define Kp 4.50f   // proportional gain governs rate of convergence to accelerometer/magnetometer
#define Ki 1.0f    // integral gain governs rate of convergence of gyroscope biases

float angles[3];
float q0, q1, q2, q3; 

void imuInit(IMU_EN_SENSOR_TYPE *penMotionSensorType)
{
	bool bRet = false;
	i2c_init(I2C_PORT,400*1000); 
	gpio_set_function(6, GPIO_FUNC_I2C);
  gpio_set_function(7, GPIO_FUNC_I2C);
  gpio_pull_up(6);
  gpio_pull_up(7);
	bRet = MPU9250_Check();
	if( true == bRet)
	{
		*penMotionSensorType = IMU_EN_SENSOR_TYPE_MPU9250;
		MPU9250_Init();
    // calibrateMagn();
	}
	else
	{
		*penMotionSensorType = IMU_EN_SENSOR_TYPE_NULL;
	}
	q0 = 1.0f;  
	q1 = 0.0f;
	q2 = 0.0f;
	q3 = 0.0f;

	return;
}
void imuDataGet(IMU_ST_ANGLES_DATA *pstAngles, 
                IMU_ST_SENSOR_DATA *pstGyroRawData,
                IMU_ST_SENSOR_DATA *pstAccelRawData,
                IMU_ST_SENSOR_DATA *pstMagnRawData)
{
  float MotionVal[9];
  
  MPU9250_READ_ACCEL();
  MPU9250_READ_GYRO();
  MPU9250_READ_MAG();

  MotionVal[0]=gyro[0]/32.8;
  MotionVal[1]=gyro[1]/32.8;
  MotionVal[2]=gyro[2]/32.8;
  MotionVal[3]=accel[0];
  MotionVal[4]=accel[1];
  MotionVal[5]=accel[2];
  MotionVal[6]=magn[0];
  MotionVal[7]=magn[1];
  MotionVal[8]=magn[2];
  imuAHRSupdate((float)MotionVal[0] * 0.0175, (float)MotionVal[1] * 0.0175, (float)MotionVal[2] * 0.0175,
                (float)MotionVal[3], (float)MotionVal[4], (float)MotionVal[5], 
                (float)MotionVal[6], (float)MotionVal[7], MotionVal[8]);


  pstAngles->fPitch = asin(-2 * q1 * q3 + 2 * q0* q2)* 57.3; // pitch
  pstAngles->fRoll = atan2(2 * q2 * q3 + 2 * q0 * q1, -2 * q1 * q1 - 2 * q2* q2 + 1)* 57.3; // roll
  pstAngles->fYaw = atan2(-2 * q1 * q2 - 2 * q0 * q3, 2 * q2 * q2 + 2 * q3 * q3 - 1) * 57.3; 

  pstGyroRawData->s16X = gyro[0];
  pstGyroRawData->s16Y = gyro[1];
  pstGyroRawData->s16Z = gyro[2];

  pstAccelRawData->s16X = accel[0];
  pstAccelRawData->s16Y = accel[1];
  pstAccelRawData->s16Z  = accel[2];

  pstMagnRawData->s16X = magn[0];
  pstMagnRawData->s16Y = magn[1];
  pstMagnRawData->s16Z = magn[2];  

  return;  
}

void imuAHRSupdate(float gx, float gy, float gz, float ax, float ay, float az, float mx, float my, float mz) 
{
  float norm;
  float hx, hy, hz, bx, bz;
  float vx, vy, vz, wx, wy, wz;
  float exInt = 0.0, eyInt = 0.0, ezInt = 0.0;
  float ex, ey, ez, halfT = 0.024f;

  float q0q0 = q0 * q0;
  float q0q1 = q0 * q1;
  float q0q2 = q0 * q2;
  float q0q3 = q0 * q3;
  float q1q1 = q1 * q1;
  float q1q2 = q1 * q2;
  float q1q3 = q1 * q3;
  float q2q2 = q2 * q2;   
  float q2q3 = q2 * q3;
  float q3q3 = q3 * q3;          

  norm = invSqrt(ax * ax + ay * ay + az * az);       
  ax = ax * norm;
  ay = ay * norm;
  az = az * norm;

  norm = invSqrt(mx * mx + my * my + mz * mz);          
  mx = mx * norm;
  my = my * norm;
  mz = mz * norm;

  // compute reference direction of flux
  hx = 2 * mx * (0.5f - q2q2 - q3q3) + 2 * my * (q1q2 - q0q3) + 2 * mz * (q1q3 + q0q2);
  hy = 2 * mx * (q1q2 + q0q3) + 2 * my * (0.5f - q1q1 - q3q3) + 2 * mz * (q2q3 - q0q1);
  hz = 2 * mx * (q1q3 - q0q2) + 2 * my * (q2q3 + q0q1) + 2 * mz * (0.5f - q1q1 - q2q2);         
  bx = sqrt((hx * hx) + (hy * hy));
  bz = hz;     

  // estimated direction of gravity and flux (v and w)
  vx = 2 * (q1q3 - q0q2);
  vy = 2 * (q0q1 + q2q3);
  vz = q0q0 - q1q1 - q2q2 + q3q3;
  wx = 2 * bx * (0.5 - q2q2 - q3q3) + 2 * bz * (q1q3 - q0q2);
  wy = 2 * bx * (q1q2 - q0q3) + 2 * bz * (q0q1 + q2q3);
  wz = 2 * bx * (q0q2 + q1q3) + 2 * bz * (0.5 - q1q1 - q2q2);  

  // error is sum of cross product between reference direction of fields and direction measured by sensors
  ex = (ay * vz - az * vy) + (my * wz - mz * wy);
  ey = (az * vx - ax * vz) + (mz * wx - mx * wz);
  ez = (ax * vy - ay * vx) + (mx * wy - my * wx);

  if(ex != 0.0f && ey != 0.0f && ez != 0.0f)
  {
    exInt = exInt + ex * Ki * halfT;
    eyInt = eyInt + ey * Ki * halfT;  
    ezInt = ezInt + ez * Ki * halfT;

    gx = gx + Kp * ex + exInt;
    gy = gy + Kp * ey + eyInt;
    gz = gz + Kp * ez + ezInt;
  }

  q0 = q0 + (-q1 * gx - q2 * gy - q3 * gz) * halfT;
  q1 = q1 + (q0 * gx + q2 * gz - q3 * gy) * halfT;
  q2 = q2 + (q0 * gy - q1 * gz + q3 * gx) * halfT;
  q3 = q3 + (q0 * gz + q1 * gy - q2 * gx) * halfT;  

  norm = invSqrt(q0 * q0 + q1 * q1 + q2 * q2 + q3 * q3);
  q0 = q0 * norm;
  q1 = q1 * norm;
  q2 = q2 * norm;
  q3 = q3 * norm;
}

float invSqrt(float x) 
{
  float halfx = 0.5f * x;
  float y = x;
  
  long i = *(long*)&y;                //get bits for floating value
  i = 0x5f3759df - (i >> 1);          //gives initial guss you
  y = *(float*)&i;                    //convert bits back to float
  y = y * (1.5f - (halfx * y * y));   //newtop step, repeating increases accuracy
  
  return y;
}



#ifdef __cplusplus
}
#endif
